/**
 * Copyright Wed Nov 13 2019
 * v1.0.2 by Igor Novikov <novikovio74@gmail.com>
 */

import seal$1 from 'core-js-pure/stable/object/seal';
import assign$1 from 'core-js-pure/stable/object/assign';
import uniqueId from 'lodash/uniqueId';
import _trimInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/trim';
import _sliceInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/slice';
import isArray$1 from 'core-js-pure/features/array/is-array';
import getIterator$1 from 'core-js-pure/features/get-iterator';
import isIterable$1 from 'core-js-pure/features/is-iterable';

var seal = seal$1;

var assign = assign$1;

var NodeTypes;

(function (NodeTypes) {
  NodeTypes[NodeTypes["Text"] = 3] = "Text";
})(NodeTypes || (NodeTypes = {}));

function isTextNode(node) {
  return node.nodeType === NodeTypes.Text;
}
function addClassName(node, className) {
  var _context;

  node.className = _trimInstanceProperty(_context = node.className + " " + className).call(_context);
}

var NodeIterator =
/*#__PURE__*/
function () {
  function NodeIterator(handler) {
    this.handler = handler;
  }

  var _proto = NodeIterator.prototype;

  _proto.iterate = function iterate(startNode, endNode, startOffset, endOffset) {
    var treeWalker = this.createTreeWalker(startNode, endNode);
    var endNodeIsText = isTextNode(endNode);
    var endNodeIndex = endOffset - 1;

    if (startNode !== endNode || startOffset !== endNodeIndex) {
      treeWalker.currentNode = isTextNode(startNode) ? this.handler(startNode, startOffset, null) : startNode.childNodes[startOffset];
      this.traverseNodeTree(treeWalker, endNodeIsText ? function (node) {
        return node !== endNode;
      } : function (node) {
        return node !== endNode.childNodes[endNodeIndex];
      });
    }

    if (endNodeIsText) {
      treeWalker.currentNode = this.handler(endNode, null, endOffset);
    } else {
      var parent = treeWalker.currentNode = endNode.childNodes[endNodeIndex];
      this.traverseNodeTree(treeWalker, function (node) {
        return parent.contains(node);
      });
    }
  };

  _proto.traverseNodeTree = function traverseNodeTree(treeWalker, condition) {
    var prevNode;

    while (prevNode !== treeWalker.currentNode && condition(treeWalker.currentNode)) {
      if (isTextNode(treeWalker.currentNode)) {
        treeWalker.currentNode = this.handler(treeWalker.currentNode, null, null);
      }

      prevNode = treeWalker.currentNode;
      treeWalker.nextNode();
    }
  };

  _proto.createTreeWalker = function createTreeWalker(startNode, endNode) {
    var range = new Range();
    range.setStart(startNode, 0);
    range.setEnd(endNode, 0);
    return document.createTreeWalker(range.commonAncestorContainer, NodeFilter.SHOW_ALL, null, false);
  };

  return NodeIterator;
}();

var isArray = isArray$1;

function _arrayWithHoles(arr) {
  if (isArray(arr)) return arr;
}

var arrayWithHoles = _arrayWithHoles;

var getIterator = getIterator$1;

var isIterable = isIterable$1;

function _iterableToArrayLimit(arr, i) {
  if (!(isIterable(Object(arr)) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = getIterator(arr), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

var iterableToArrayLimit = _iterableToArrayLimit;

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var nonIterableRest = _nonIterableRest;

function _slicedToArray(arr, i) {
  return arrayWithHoles(arr) || iterableToArrayLimit(arr, i) || nonIterableRest();
}

var slicedToArray = _slicedToArray;

var NodeReplacer =
/*#__PURE__*/
function () {
  NodeReplacer.expandElement = function expandElement(element) {
    var parent = element.parentNode;
    element.insertAdjacentHTML("afterend", element.innerHTML);
    parent.removeChild(element);
    parent.innerHTML = parent.innerHTML; // collapse text nodes
  };

  function NodeReplacer(options) {
    var _this = this;

    this.options = options;

    this.replaceTextNode = function (node, startOffset, endOffset) {
      var _this$options = _this.options,
          id = _this$options.id,
          className = _this$options.className;

      var _this$breakTextConten = _this.breakTextContent(node, startOffset, endOffset),
          _this$breakTextConten2 = slicedToArray(_this$breakTextConten, 3),
          firstPartOfString = _this$breakTextConten2[0],
          stringToBeWrapped = _this$breakTextConten2[1],
          lastPartOfString = _this$breakTextConten2[2];

      if (stringToBeWrapped) {
        var fragment = document.createDocumentFragment();

        if (firstPartOfString) {
          fragment.appendChild(document.createTextNode(firstPartOfString));
        }

        var wrapper = document.createElement("span");
        wrapper.innerHTML = stringToBeWrapped;
        wrapper.setAttribute(NodeReplacer.attribute, id);
        if (className) addClassName(wrapper, className);
        fragment.appendChild(wrapper);

        if (lastPartOfString) {
          fragment.appendChild(document.createTextNode(lastPartOfString));
        }

        var lastTextNode = fragment.lastChild.firstChild || fragment.lastChild;

        _this.replaceTextNodeWithBrokenString(node, fragment);

        return lastTextNode;
      }

      return node;
    };
  }

  var _proto = NodeReplacer.prototype;

  _proto.replaceTextNodeWithBrokenString = function replaceTextNodeWithBrokenString(textNode, fragment) {
    var parentNode = textNode.parentNode;
    parentNode.insertBefore(fragment, textNode);
    parentNode.removeChild(textNode);
  };

  _proto.breakTextContent = function breakTextContent(node, startOffset, endOffset) {
    var content = node.textContent;
    startOffset = startOffset || 0;
    endOffset = endOffset || content.length;
    return [_sliceInstanceProperty(content).call(content, 0, startOffset), _sliceInstanceProperty(content).call(content, startOffset, endOffset), _sliceInstanceProperty(content).call(content, endOffset, content.length)];
  };

  return NodeReplacer;
}();

NodeReplacer.attribute = "data-id";

var Highlight =
/*#__PURE__*/
function () {
  function Highlight() {}

  var _proto = Highlight.prototype;

  _proto.fromSelection = function fromSelection(options) {
    var id = null;
    var selection = window.getSelection();

    if (!selection.isCollapsed) {
      id = this.fromRange(selection.getRangeAt(0), options);
      this.selectHighlightedRange(id, selection);
    }

    return id;
  };

  _proto.fromRange = function fromRange(range, options) {
    return !range.collapsed ? this.fromNodes(range.startContainer, range.endContainer, range.startOffset, range.endOffset, options) : null;
  };

  _proto.fromNodes = function fromNodes(startNode, endNode, startOffset, endOffset, options) {
    if (startNode instanceof Node && endNode instanceof Node) {
      options = this.getSafeOptions(options);
      var nodeReplacer = new NodeReplacer(options);

      if (startNode === endNode && isTextNode(startNode)) {
        nodeReplacer.replaceTextNode(startNode, startOffset, endOffset);
      } else {
        var nodeIterator = new NodeIterator(nodeReplacer.replaceTextNode);
        nodeIterator.iterate(startNode, endNode, startOffset, endOffset);
      }

      return options.id;
    }

    return null;
  };

  _proto.remove = function remove(id) {
    var wrapper;

    while (wrapper = document.querySelector("[" + NodeReplacer.attribute + "=\"" + id + "\"]")) {
      NodeReplacer.expandElement(wrapper);
    }
  };

  _proto.getSafeOptions = function getSafeOptions(options) {
    return assign(seal({
      id: uniqueId(),
      className: "highlight"
    }), options);
  };

  _proto.selectHighlightedRange = function selectHighlightedRange(id, selection) {
    var highlightedNodes = document.querySelectorAll("[" + NodeReplacer.attribute + "=\"" + id + "\"]");
    var range = new Range();
    range.setStartBefore(highlightedNodes[0]);
    range.setEndAfter(highlightedNodes[highlightedNodes.length - 1]);
    selection.removeAllRanges();
    selection.addRange(range);
  };

  return Highlight;
}();

export default Highlight;
//# sourceMappingURL=highlight.esm.js.map
